%{

/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


#include <stdlib.h>
#include <string.h>
  
extern "C" {
  int yywrap() ;
  int yyparse() ;
}

extern FILE* yyin ;
extern FILE* yyout ;
extern int bracket_count ;
extern int linecount ;

#define AFTER_PRAGMA 14

int newline(char*) ;

class SchHead ;
#include "class_y.h"

static int first_time = 1 ;

%}

%s S_NORMAL S_SCHCLASS S_BODY S_CMT S_CMT_CLASS S_CMT_BODY 

IDENTIFIER      [a-zA-Z_][a-zA-Z_0-9]*
ACCESS          public|private|protected
VIRTUAL         virtual
SCHCLASS        [ \n\t]?class[ \n\t]?
SCHCLASSDECL    [ \n\t]?class[ \n\t]?[a-zA-Z_][a-zA-Z_0-9]*[ \n\t]*;
OBRACKET        \{
CBRACKET        \}
CXX_CMT         \/\/.*\n
C_BEGIN_CMT     \/\*
C_END_CMT       \*\/
EMPTYPARAM      \([ \n\t]*\)

%%

%{
  if (first_time == 1) {
    BEGIN S_NORMAL ;
    first_time = 0 ;
  }
%}

[ \t]           {
                  ECHO ;
		}

[\n]            {
                  ECHO ;
                  linecount++ ;
                }

{CXX_CMT}       {
                  ECHO ;
                  linecount++ ;
		}

<S_NORMAL>{C_BEGIN_CMT}   {
                  ECHO ;
		  BEGIN S_CMT ;
		}

<S_CMT>{C_END_CMT} {
                  ECHO ;
		  BEGIN S_NORMAL ;
		}

<S_CMT>.        {
                  ECHO ;
		}

<S_SCHCLASS>{C_BEGIN_CMT}   {
                  ECHO ;
		  BEGIN S_CMT_CLASS ;
		}

<S_CMT_CLASS>{C_END_CMT} {
                  ECHO ;
		  BEGIN S_SCHCLASS ;
		}

<S_CMT_CLASS>.    {
                  ECHO ;
		}

<S_BODY>{C_BEGIN_CMT}   {
                  ECHO ;
		  BEGIN S_CMT_BODY ;
		}

<S_CMT_BODY>{C_END_CMT} {
                  ECHO ;
		  BEGIN S_BODY ;
		}

<S_CMT_BODY>.    {
                  ECHO ;
		}

<S_NORMAL>{SCHCLASSDECL}  {
                  ECHO ;
		  return (SCHCLASSDECL) ;
		}

<S_NORMAL>{SCHCLASS}      {
                  fprintf(yyout, (char*)yytext) ;
      	          linecount += newline((char*)yytext) ;
                  BEGIN S_SCHCLASS ;
		  return(SCHCLASS);
                }

<S_BODY>{OBRACKET}      {
                  ECHO; 
		  bracket_count++ ;
		}

<S_BODY>{CBRACKET}      {
                  strcpy(yylval.ident, (char*)yytext);		  
		  ECHO ;
		  if (--bracket_count == 1) {		    
		    BEGIN S_SCHCLASS ;
		  } 
		}

<S_BODY>.       { 
                  ECHO ;
		}

<S_SCHCLASS>{OBRACKET}      {
                  strcpy(yylval.ident, (char*)yytext);
		  if (bracket_count++ != 0) {
		    ECHO ;
		    BEGIN S_BODY ;
		  } 
		  return OBRACKET ;
		}

<S_SCHCLASS>{CBRACKET}      {
                  bracket_count-- ;
		  if (bracket_count == 0) {
		    BEGIN S_NORMAL ;		    
		  }
                  return CBRACKET ;
		}


<S_SCHCLASS>{ACCESS}         {
                  ECHO ;
		  return(ACCESS);
                }

<S_SCHCLASS>{VIRTUAL}         {
                  ECHO ;
		  return(VIRTUAL);
                }

<S_SCHCLASS>{EMPTYPARAM}      {
                  ECHO ;
		  return(EMPTYPARAM);
                }

<S_SCHCLASS>ListX         {
                  fprintf(yyout, "LocalListX") ; 
		  return(LISTX);
                }

<S_SCHCLASS>{IDENTIFIER}    {
                  ECHO ;
                  strcpy(yylval.ident, (char*)yytext);
		  return(IDENTIFIER);
                }

<S_SCHCLASS>.   { 
                  ECHO ;
                  return yytext[0];
		}


%%

int newline(char* str) {

  int cpt = 0 ;
  while ( (str = index(str,'\n')) != NULL) {
    cpt++ ;
    str++ ;
  }
 
  return cpt;
}
